

//	#define TMPOUT


#include "cpsp/gecode/GC_ThreadingSpace.hh"
#include "cpsp/gecode/GC_StlSetRangeIterator.hh"
#include "cpsp/gecode/GC_LatticeNeighbored2.hh"
//#include "cpsp/gecode/GC_Neighbored2.hh"
//#include "cpsp/gecode/GC_Neighbored3.hh"
#include "cpsp/gecode/GC_ValNearCenter.hh"

#include <gecode/kernel.hh>
#include <gecode/int.hh>
#include <gecode/search.hh>
#include <gecode/minimodel.hh>
#include <gecode/int/branch.hh>

#include <ell/LatticeFrame.hh>



#ifdef TMPOUT
	#include <iostream>
#endif

namespace cpsp
{
  namespace gecode
  {
  	
  		// hilfsfunktion ... TODO : loeschen ;)
	void print(ell::IndexSet& x, const ell::LatticeFrame* latFrame) {
		std::cout<<"( ";
		for (ell::IndexSet::const_iterator it = x.begin(); it != x.end(); it++)
			std::cout <<latFrame->getPoint(*it) <<"| ";
		std::cout <<")";
		std::cout.flush();
	}
  	
  	
	GC_ThreadingSpace::GC_ThreadingSpace( HPThreadingOptions& opts, 
			HCore& hCore) :
		domains(	this, opts.getSequence()->size(), 
					0, Gecode::Limits::Int::int_max)
	{
			// variablen initialisieren
		ell::IndexSet data;
		GC_StlSetRangeIterator dataIt;
		Gecode::IntSet dataSet;
		
			// frame size setzen
		opts.setLatticeFrameSize(	opts.getMaxPCoreDistance() 
									+ hCore.getMaxDimension());
		 // TODO : test obs hieran liegt
//		opts.setLatticeFrameSize(	100 );
		
			// domain init
		const std::vector<unsigned int>*const hullLvl = opts.getHullLevel();
		
		int i=0;
		
#ifdef TMPOUT
	std::cout	<<" seq  = " <<(*opts.getSequence())
				<<std::endl
				<<" hull = ";
	for (i=0; i<hullLvl->size(); i++) std::cout <<hullLvl->at(i);
	std::cout <<std::endl;
#endif
		
		for (i=0; i<domains.size(); i++) {
				// daten holen
			data = hCore.getIndexedHull(opts.getLattice(), hullLvl->at(i));
				// iterator erzeugen
			dataIt.init(&data);
				// daten einschraenken auf werte in dataIt via IntView
			Gecode::Int::IntView actDom(domains[i]);
				// macro zu sicherheit, wenn schnittmengenbildung schief laeuft
			GECODE_ME_FAIL(this, actDom.inter(this,dataIt));
		}
		
			// p-singlet init
		std::vector<unsigned int>::const_iterator it;
			// daten holen
		data = hCore.getIndexedPSinglets(opts.getLattice());
			// iterator erzeugen
		dataIt.init(&data);
			// domains einschraenken
		for (	it = opts.getPSinglets().begin(); 
				it != opts.getPSinglets().end(); it++) 
		{
				// zugriffs-view auf singlet domain
			Gecode::Int::IntView actDom(domains[*it]);
				// macro zu sicherheit, wenn schnittmengenbildung schief laeuft
			GECODE_ME_FAIL(this, actDom.inter(this,dataIt));
		}
/* funzt aber bringt kaum was		
			// h-singlet init
			// daten holen
		data = hCore.getIndexedHSinglets(opts.getLattice());
			// iterator erzeugen
		dataIt.init(&data);
			// domains einschraenken
		for (	it = opts.getHSinglets().begin(); 
				it != opts.getHSinglets().end(); it++) 
		{
				// zugriffs-view auf singlet domain
			Gecode::Int::IntView actDom(domains[*it]);
				// macro zu sicherheit, wenn schnittmengenbildung schief laeuft
			GECODE_ME_FAIL(this, actDom.inter(this,dataIt));
		}
	*/	
			// constraints initialisieren
			// ALL DIFFERENT 
			// Supports value (ICL_VAL, default), bounds (ICL_BND), 
			// and domain-consistency (ICL_DOM)
		Gecode::distinct(this, domains, Gecode::ICL_DOM);

			// propagationsstufen fuer nachbarschaftsconstraints
/*		unsigned int maxBinPropSize = 0, maxTerPropSize = 0;

			// maximale domain propagationsgroesse auf groesse der
			// domains der zweiten huelle gesetzt
		maxBinPropSize = hCore.getIndexedHull(opts.getLattice(),2).size();
			// maximale domain propagationsgroesse auf groesse der
			// domains der kernvariablen gesetzt
		maxTerPropSize = hCore.getIndexedHull(opts.getLattice(),0).size();
		
			// sollte nie auftreten.. ;) --> einfach vertauschen
		if (maxBinPropSize < maxTerPropSize) {
			maxBinPropSize = maxTerPropSize;
			maxTerPropSize = hCore.getIndexedHull(opts.getLattice(),2).size();
		}
*/
			// binaere neighbor constraints erzeugen
		for(i=1; i<domains.size(); i++) {
			GECODE_ES_FAIL(this,
			GC_LatticeNeighbored2::post(	this, 
									domains[i-1], domains[i], 
									opts.getLattice(),
									Gecode::ICL_BND
//									,maxBinPropSize,
//									false,
//									maxTerPropSize
								));
		}

/*			// ternaere neighbor constraints erzeugen
			// setzen da mit propagation ein, wo binaere aufhoeren
		for(i=2; i<domains.size(); i++) {
			GECODE_ES_FAIL(this,
			GC_Neighbored3::post(	this, 
									domains[i-2], domains[i-1], domains[i], 
									opts.getLattice(),
									Gecode::ICL_BND,
									maxTerPropSize,
									true));
		}
*/
			// branching festlegen
//		branch(this, x, Gecode::BVAR_SIZE_MIN, Gecode::BVAL_MIN);
	
			// viewarray erzeugen
		Gecode::ViewArray<Gecode::Int::IntView> xv(this,(Gecode::IntVarArgs)domains);
			// GC_ValNearCenter lattice frame initialisieren
		GC_ValNearCenter::latFrame = opts.getLattice();
			// branching erzeugen
		new (this) Gecode::ViewValBranching<	Gecode::Int::IntView,
												int,
												Gecode::Int::Branch::ByMinMin,
												GC_ValNearCenter
											>(this, xv);
	}
	
	
	
	GC_ThreadingSpace::GC_ThreadingSpace(bool share, GC_ThreadingSpace& toCopy):
		Gecode::Space(share, toCopy), domains()
	{
		domains.update(this, share, toCopy.domains);
		
	}
	
	
	GC_ThreadingSpace::~GC_ThreadingSpace()
	{
	}
	
	
	Gecode::Space*
	GC_ThreadingSpace::copy(bool share) {
		return new GC_ThreadingSpace(share, *this);
	}
	
	ell::IndexVec
	GC_ThreadingSpace::getSolution() const {
		ell::IndexVec retVec;
		for (int i=0; i<domains.size();i++) {
			retVec.push_back((ell::LatticeFrame::index_type)domains[i].val());
		}
		return retVec;
	}
	

  } // namespace gecode
}  // namespace cpsp
